from .router.cli.md_tn5916 import tn5916_v3
from .router.cli.rp_api.rp_api import Router
from .router.cli.md_edrg9010 import edrg9010_v2
from .router.cli.md_edrg9010 import edrg9010_v3

from .router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000